/*
 * AudioFile.h
 *
 *  Created on: 6 déc. 2023
 *      Author: fanch
 */

#ifndef AUDIOFILE_H_
#define AUDIOFILE_H_
#include <string>
#include <cstdint>
#include <cstdio>

class Window;

struct AudiFileHeader {
	char riff[4]; // 0
	uint32_t size; // 4
	char wave[4]; // 8
	char fmt[4]; // 12
	uint32_t length; // 16
	uint16_t type; // 20
	uint16_t channels; // 22
	uint32_t sample_rate; // 24
	uint32_t byte_per_sec; // 28
	uint16_t __; // 32
	uint16_t bit_per_sample; // 34
	char data[4]; // 36
	uint32_t data_size; //40
};

struct AudioFileOption {
	float time_per_window;
	uint32_t buffer_size;

};


class AudioFile {
public:
	virtual ~AudioFile();
	AudioFile(FILE* fd=NULL);
	AudioFile(const std::string&);
	AudioFileOption& get_options() { return m_options; }
	AudiFileHeader& get_headers() { return m_header; }


	Window** window_at_time(float time);
	Window* window_channel_at_time(float time, int ch);
	inline size_t sample_per_window(){
		return m_options.time_per_window * m_header.bit_per_sample * m_header.sample_rate;
	}

	float m_total_time;

protected:
	FILE* m_fd;
	AudiFileHeader m_header;
	AudioFileOption m_options;
	int8_t* m_buffer;
	size_t m_buffer_start;
	size_t m_buffer_end;


	int8_t* _bufferize(size_t start);

	void _init();
	char* _base_read(size_t size, size_t count, char* buffer=NULL);



};

inline double _val_to_float(int8_t* val, int size){
    float fval;
    switch (size) {
        case 1: fval=((uint8_t)*val)/((double)255); break;
        case 2: fval=(*(int16_t*)val)/((double)32767.0); break;
        case 3: {
        	throw "Error not implemented";
        	uint32_t v = *(uint32_t*)(val);
        	fval=((v & (1<<24)))-(v&((1<<24)-1)); break;
        }
        case 4: fval=(*(int32_t*)val)/((double)(1<<31)); break;
        default:
            fprintf(stderr, "Erreur impossible de traiter des échantillons sur %d bits", size);
            throw "Error";

    }
    return (fval/2.0f)-((float)(1<<(size-1)));
}

#endif /* AUDIOFILE_H_ */
