/*
 * PluginHost.cpp
 *
 *  Created on: 17 déc. 2023
 *      Author: fanch
 */

#include "PluginIndex.h"

#include <dirent.h>
#include <dlfcn.h>
#include <cstring>
#include <cstdlib>
#include <sys/types.h>
#include <unistd.h>
#include "common/tools.h"

DllManager* DllManager::m_instance = new DllManager();

void PluginEntry::load(){
	void * plugin_handle;
	LADSPA_Descriptor_Function descriptor;
	m_loaded = false;
	plugin_handle =  DllManager::open(m_path);
	if(plugin_handle){
		dlerror();
		descriptor = (LADSPA_Descriptor_Function)dlsym(plugin_handle, "ladspa_descriptor");
		const char error = dlerror() != NULL;
		m_is_error = error;
		if(!error){
			int i=0;
			const LADSPA_Descriptor* desc;
			for(int i=0; (desc=descriptor(i)); i++){
				m_name = desc->Name;
				m_id = desc->UniqueID;
				m_label = desc->Label;
				m_ports.resize(desc->PortCount);
				for(int i=0; i<desc->PortCount; i++){
					m_ports[i].name = desc->PortNames[i];
					m_ports[i].index = i;
					LADSPA_PortDescriptor port_desc = desc->PortDescriptors[i];
					m_ports[i].is_input = LADSPA_IS_PORT_INPUT(port_desc);
					m_ports[i].is_output = LADSPA_IS_PORT_OUTPUT(port_desc);
					m_ports[i].is_audio = LADSPA_IS_PORT_AUDIO(port_desc);
					m_ports[i].is_control = LADSPA_IS_PORT_CONTROL(port_desc);
				}
			}

			DllManager::close(plugin_handle);
		}else{
			m_error = error;
			DllManager::close(plugin_handle);
			return;
		}
		m_loaded = true;
	}

}

void PluginIndex::list() const{

}

void PluginIndex::_update_dir(const std::string& dir){
	const char * directory = dir.c_str();
	char * full_path;
	DIR * psDirectory;
	long dir_len;
	long iNeedSlash;
	struct dirent * direntry;
	void * pvPluginHandle;

	dir_len = strlen(directory);
	if (!dir_len) return;
	if (directory[dir_len - 1] == '/')
		iNeedSlash = 0;
	else
		iNeedSlash = 1;

	psDirectory = opendir(directory);
	if (!psDirectory) return;

	while (1) {
		direntry = readdir(psDirectory);
		if (!direntry) {
		  closedir(psDirectory);
		  return;
		}
		printf("-> %s\n", direntry->d_name);
		if(!endswith(direntry->d_name, ".so")){
			continue;
		}

		full_path = (char*)new char[(dir_len+ strlen(direntry->d_name)+ 1 + iNeedSlash)];
		strcpy(full_path, directory);
		if (iNeedSlash)  strcat(full_path, "/");
		strcat(full_path, direntry->d_name);

		PluginEntry entry(full_path);
		m_plugins.push_back(entry);
		m_plugins[m_plugins.size()-1].load();
		delete[] full_path;
	}
}


void PluginIndex::update(){
	m_plugins.clear();
	for(size_t i=0; i<m_plugin_pathes.size(); i++){
		_update_dir(m_plugin_pathes[i]);
	}
}

PluginEntry& PluginIndex::load_from_path(const std::string& path){
	for(size_t i=0; i<m_plugins.size(); i++){
		if(m_plugins[i].path() == path){
			return m_plugins[i];
		}
	}
	PluginEntry entry(path);
	m_plugins.push_back(entry);
	return m_plugins[m_plugins.size() -1 ];
}
