/*
 * PluginInstance.cpp
 *
 *  Created on: 17 déc. 2023
 *      Author: fanch
 */

#include "PluginManager.h"
#include "PluginIndex.h"
#include "PluginInstance.h"

PluginInstance::PluginInstance(PluginManager& manager, PluginEntry& entry, int index, int id) :
	m_id(id), m_index(index), m_manager(manager), m_entry(entry){
	m_dll = DllManager::open(entry.path());
	if(!m_dll){
		throw "unable to open dll";
	}
	LADSPA_Descriptor_Function fct = (LADSPA_Descriptor_Function)dlsym(m_dll, "ladspa_descriptor");
	if( dlerror() != NULL){
		throw "unable to open dll 'ladspa_descripto'";
	}
	m_desc =  (LADSPA_Descriptor*) fct(index);

	if( m_desc == NULL){
		throw "unable to open dll 'ladspa_descripto' desc";
	}
	m_sample_rate = manager.sample_rate();
	m_instance = m_desc->instantiate(m_desc, m_sample_rate);
}

PluginInstance::~PluginInstance(){
	DllManager::close(m_dll);
}


void PluginInstance::disconnect(int id){
	if(id<m_entry.m_ports.size()){
		m_desc->connect_port(m_instance, id, NULL);
	}
}

void PluginInstance::connect(int port_id, LADSPA_Data* data){
	if(port_id<m_entry.m_ports.size()){
		m_desc->connect_port(m_instance, port_id, data);
	}
}

LADSPA_Data PluginInstance::get(int id) const{
	return 0.0;
}

void PluginInstance::set(int id, LADSPA_Data d){

}

void PluginInstance::activate(){
	m_desc->activate(m_instance);
}

void PluginInstance::desactivate(){
	m_desc->deactivate(m_instance);

}

void PluginInstance::run(){
	m_desc->run(m_instance, m_sample_rate);

}
