/*
 * PluginInstance.h
 *
 *  Created on: 17 déc. 2023
 *      Author: fanch
 */

#ifndef SRC_CORE_PLUGININSTANCE_H_
#define SRC_CORE_PLUGININSTANCE_H_

#include <ladspa.h>

#include <vector>

class PluginInstance;

struct PluginConnection {

	PluginConnection(PluginInstance *s, PluginInstance *d, LADSPA_Data *b) :
			src(s), dst(d), buffer(b) {
	}

	PluginConnection(const PluginConnection &o) {
		src = o.src;
		dst = o.dst;
		buffer = o.buffer;
	}

	PluginInstance *src;
	PluginInstance *dst;
	LADSPA_Data *buffer;
};

typedef std::vector<PluginConnection> PluginConnections;

class PluginManager;
class PluginEntry;
class PluginPort;

class PluginInstance {

	PluginInstance(PluginManager &manager, PluginEntry &entry, int index,
			int id);

	virtual ~PluginInstance();
	void disconnect(int id);
	void connect(int self_id, LADSPA_Data *data);
	LADSPA_Data get(int id) const;
	void set(int id, LADSPA_Data d);
	void activate();
	void desactivate();
	void run();

	PluginEntry& entry() {
		return m_entry;
	}

protected:

	int m_id;
	int m_index;
	LADSPA_Handle m_instance;
	LADSPA_Descriptor* m_desc;
	void *m_dll;
	PluginManager &m_manager;
	PluginEntry &m_entry;
	unsigned long m_sample_rate;

	friend class PluginManager;
};

#endif /* SRC_CORE_PLUGININSTANCE_H_ */
