/*
 * PluginManager.cpp
 *
 *  Created on: 17 déc. 2023
 *      Author: fanch
 */

#include "PluginManager.h"
#include "PluginIndex.h"
#include "PluginInstance.h"


PluginManager::~PluginManager(){
	for(LADSPA_Data* elem : m_buffers ){
		delete[] elem;
	}
	for(PluginInstance* elem : m_plugins ){
		delete elem;
	}
	m_buffers.clear();
	if(m_input) delete[] m_input;
	if(m_output) delete[] m_input;
}

void PluginManager::connect(PluginInstance *p1, int p1id, PluginInstance *p2,
		int p2id) {
}

void PluginManager::disconnect(PluginInstance *p, int pid) {

}



LADSPA_Data* PluginManager::run(LADSPA_Data* input){
	if(m_plugins.size()){
		_copy_buffer(input, m_input);
		for(PluginInstance* plugin : m_plugins){
			plugin->run();
		}
	}else{
		_copy_buffer(input, m_output);
	}
	return m_output;
}

PluginInstance* PluginManager::add_plugin(PluginEntry &p) {
	if(!m_input || !m_output){
		throw ("Erreur non initialisé");
	}
	LADSPA_Data *input = m_input, *output=m_output;
	PluginInstance* plugin = new PluginInstance(*this, p, m_plugins.size(), m_current_id);
	m_current_id += 1;

	if(m_plugins.size()){
		input = new LADSPA_Data[m_sample_count];
		m_buffers.push_back(input);
		PluginInstance* last = m_plugins[m_plugins.size()-1];
		for(const PluginPort& port : last->entry().ports()){
			last->connect(m_buffers.size()-1, input);
		}
	}
	for(const PluginPort& port : p.ports()){
		if(port.is_audio){
			if(port.is_input){
				plugin->connect(port.index, output);
			}
			if(port.is_output){
				plugin->connect(port.index, output);
			}
		}
	}
	m_plugins.push_back(plugin);
	return plugin;
}

LADSPA_Data* PluginManager::_copy_buffer(LADSPA_Data* src, LADSPA_Data* dst) {
	for(unsigned long i=0; i<m_sample_count; i++){
		dst[i] = src[i];
	}
	return dst;
}
