/*
 * PluginManager.h
 *
 *  Created on: 17 déc. 2023
 *      Author: fanch
 */

#ifndef SRC_CORE_PLUGINMANAGER_H_
#define SRC_CORE_PLUGINMANAGER_H_

#include <ladspa.h>

#include <vector>

class PluginInstance;
class PluginEntry;


class PluginManager {
public:
	PluginManager() : m_sample_count(0), m_sample_rate(0){
		m_input = m_output = NULL;
		m_current_id = 1;
	}

	void set_rate(int sr, int sc=1024){
		m_sample_rate = sr;
		m_sample_count = sc;
		if(m_input) delete[] m_input;
		if(m_output) delete[] m_input;
		m_input = new LADSPA_Data[m_sample_count];
		m_output = new LADSPA_Data[m_sample_count];
	}

	virtual ~PluginManager();
	unsigned long sample_rate() const{
		return m_sample_rate;
	}



	void connect(PluginInstance* p1, int p1id, PluginInstance* p2, int p2id);
	void disconnect(PluginInstance* p, int pid);
	PluginInstance*  add_plugin(PluginEntry& p);
	LADSPA_Data* run(LADSPA_Data*);



protected:

	LADSPA_Data* _copy_buffer(LADSPA_Data* src, LADSPA_Data* dst);

	unsigned long m_sample_rate;
	unsigned long m_sample_count;
	LADSPA_Data* m_input;
	LADSPA_Data* m_output;
	std::vector<LADSPA_Data*> m_buffers;
	std::vector<PluginInstance*> m_plugins;
	unsigned int m_current_id;



};

#endif /* SRC_CORE_PLUGINMANAGER_H_ */
