/*
 * FFT.h
 *
 *  Created on: 12 déc. 2023
 *      Author: fanch
 */

#ifndef FFT_H_
#define FFT_H_
#include <cstdint>
#include <cstdlib>
#include <cstdio>
#include <cmath>
#include <cstddef>
#include <fftw3.h>

class RealFFT {
public:
	RealFFT(size_t count, uint32_t rate) : m_count(count), m_rate(rate), m_input(NULL), m_output(NULL){
		m_input = new double[count];
		m_output= new double[count];
	}

	virtual ~RealFFT(){
		if(m_input){
			delete[] m_input;
		}
		if(m_output){
			delete[] m_output;
		}
	}

	void execute(double* input){
		_set_input(input);
		process();
	}

	virtual void plot(const char*filename){
		FILE* f = fopen(filename,"w");
		for(size_t i=0; i<m_count; i++){
			fprintf(f, "%ld %lf\n", i, m_input[i]);
		}
		fclose(f);
	}

protected:
	size_t m_count;
	uint32_t m_rate;
	double* m_input;
	double* m_output;

	virtual void _set_input(double* input) = 0;

	virtual void process() = 0;

};


class RealFFTW : public RealFFT{

public:
	RealFFTW(size_t count, uint32_t rate) : RealFFT(count, rate), m_input_complex(NULL), m_output_complex(NULL){
		m_input_complex = fftw_alloc_complex(count);
		m_output_complex = fftw_alloc_complex(count);
	}


	virtual void plot(const char*filename){
		FILE* f = fopen(filename,"w");
		for(size_t i=0; i<m_count; i++){
			double d = m_output[i];
			fprintf(f, "%f %lf\n", i*m_rate/((float)m_count), d>=0?d:-d);
		}
		fclose(f);
	}

protected:
	fftw_complex* m_input_complex;
	fftw_complex* m_output_complex;
	virtual void process(){
		fftw_plan p = fftw_plan_dft_1d(m_count, m_input_complex, m_output_complex, FFTW_FORWARD, FFTW_ESTIMATE);
		fftw_execute(p);
		fftw_destroy_plan(p);

		for(size_t i=0; i<m_count; i++){
			m_output[i] = m_output_complex[i][0];
		}
	}



	virtual void _set_input(double* input){
		for(size_t i=0; i<m_count; i++){
			m_input[i] = input[i];
			m_input_complex[i][0] = input[i];
			m_input_complex[i][1] = 0.0;
		}
	}
};


#endif /* FFT_H_ */
